package org.relaxng.datatype;

import java.util.Map;
import java.util.Vector;
import java.util.Iterator;

/**
 * creates an user-defined type by adding parameters to the pre-defined type.
 * 
 * @author <a href="mailto:kohsuke.kawaguchi@eng.sun.com">Kohsuke KAWAGUCHI</a>
 */
public interface DatatypeBuilder {
	
	/** adds a new parameter.
	 *
	 * @param name
	 *		the name of the parameter to be added.
	 * @param strValue
	 *		the raw value of the parameter. Caller may not normalize this value.
	 * @param context
	 *		The context information which can be used by the implementation to
	 *		acquire additional information. This context object is valid only
	 *		during this method call. The implementations may not keep the reference
	 *		to this object.
	 * @exception	DatatypeException
	 *		when the given parameter is inappropriate for some reason.
	 *		The implementation is responsible to recover from this error.
	 *		That is, the object should behave as if no such error was occured.
	 */
	void add( String name, String strValue, ValidationContext context )
		throws DatatypeException;
	
	/**
	 * derives a new Datatype from a Datatype by parameters that were already set.
	 * 
	 * @exception DatatypeException
	 *		DatatypeException is thrown if the derivation is somehow invalid.
	 *		For example, required parameter is missing, etc.
	 */
	Datatype derive() throws DatatypeException;
}
