package org.relaxng.datatype;

/**
 * signals Datatype related exceptions.
 * 
 * @author <a href="mailto:kohsuke.kawaguchi@eng.sun.com">Kohsuke KAWAGUCHI</a>
 */
public class DatatypeException extends Exception {
	
	public DatatypeException( Datatype type, int index, String msg ) {
		super(msg);
		this.type = type;
		this.index = index;
	}
	public DatatypeException( String msg ) {
		this(null,UNKNOWN,msg);
	}
	/**
	 * a constructor for those datatype libraries which don't support any
	 * diagnostic information at all.
	 */
	public DatatypeException() {
		this(null,UNKNOWN,null);
	}

	private final Datatype type;
	
	/** gets the Datatype object that generates this error. */
	public Datatype getDatatype() {
		return type;
	}
	
	
	private final int index;
	
	public static int UNKNOWN = -1;

	/**
	 * gets the index of the content where the error occured.
	 * UNKNOWN can be returned to indicate that no index information is available.
	 */
	public int getIndex() {
		return index;
	}
}
