package org.relaxng.datatype;

/**
 * Datatype streaming validator.
 * 
 * The streaming validator is an optional feature that is useful for
 * certain Datatypes. It allows the caller to incrementally provide
 * the literal.
 * 
 * @author <a href="mailto:kohsuke.kawaguchi@eng.sun.com">Kohsuke KAWAGUCHI</a>
 */
public interface DatatypeStreamingValidator {
	
	/**
	 * let a streaming validator process additional literal fragment.
	 * 
	 * <p>
	 * The application can call this method several times, then call
	 * the isValid method to check the validity.
	 */
	void characters( char[] buf, int start, int len );
	
	/**
	 * gets if the accumulated literal is valid with respect to
	 * the underlying Datatype. The application should call it
	 * when there is no more characters.
	 * 
	 * <p>
	 * This method can be called only once, and this object
	 * should be discarded after that.
	 * 
	 * @return
	 *		True if it is valid. False if otherwise.
	 */
	boolean finish();
	
	/**
	 * similar to the finish method, but this method throws
	 * Exception (with possibly diagnostic information), instad of
	 * returning false.
	 * 
	 * <p>
	 * If this method returns without any exception thrown, that means
	 * the accumulated string is valid.
	 * 
	 * @exception DatatypeException
	 *		If the callee supports the diagnosis and the accumulated literal is invalid,
	 *		then this exception that possibly contains diagnosis information is thrown.
	 * 
	 */
	void check() throws DatatypeException;
}
