package org.relaxng.datatype.helpers;

import java.util.Iterator;
import sun.misc.Service;
import org.relaxng.datatype.DatatypeLibrary;

/**
 * factory class for the DatatypeLibrary class.
 * 
 * <p>
 * The application can call the find method to automatically find
 * the appropriate datatype library for a particular URI.
 * 
 * <p>
 * The datatype library should extend this class if it wants to be
 * found by the schema processors.
 * The implementor also have to place a file in your jar file.
 * See the reference datatype library implementation for detail.
 * 
 * <p>
 * Note that this class uses non-standard class "sun.misc.Service".
 * 
 * @author <a href="mailto:kohsuke.kawaguchi@eng.sun.com">Kohsuke KAWAGUCHI</a>
 */
public abstract class DatatypeLibraryFactory {
	
	/**
	 * creates a new instance of a DatatypeLibrary that supports 
	 * the specified namespace URI.
	 * 
	 * @return
	 *		null if the specified namespace URI is not supported.	
	 */
	protected abstract DatatypeLibrary create( String namespaceURI );
	
	/**
	 * searches the available datatype libraries that
	 * supports the particular URI.
	 * 
	 * @return
	 *		If a datatype library is available, then a reference to
	 *		the object is returned. If it is not available, it returns null.
	 */
	public static DatatypeLibrary find( String namespaceURI ) {
		Iterator itr = Service.providers( DatatypeLibraryFactory.class );
		while(itr.hasNext()) {
			DatatypeLibraryFactory factory = (DatatypeLibraryFactory)itr.next();
			try {
				DatatypeLibrary lib = factory.create(namespaceURI);
				if(lib!=null)	return lib;
			} catch (Throwable t) {}	// ignore any error
		}
		return null;
	}
}
